@@ -10,7 +10,6 @@ Following the principle: "A single shell centralizes initialization and teardown
 
 import asyncio
 import logging
-import sys
 from contextvars import ContextVar
 from pathlib import Path
 from typing import Any, Awaitable, Callable, Dict, Optional
@@ -19,7 +18,6 @@ try:
     from mcp import LoggingLevel
     from mcp.types import (
         LATEST_PROTOCOL_VERSION,
-        CompletionCapability,
         Implementation,
         InitializeResult,
         LoggingCapability,
@@ -31,7 +29,8 @@ try:
 
     _MCP_TYPES_AVAILABLE = True
     _PROTOCOL_VERSION = LATEST_PROTOCOL_VERSION
-    _SUPPORTED_LOG_LEVELS = list(LoggingLevel.__args__)
+    # Use fallback log levels since LoggingLevel.__args__ may not be available
+    _SUPPORTED_LOG_LEVELS = ["debug", "info", "warning", "error"]
 except Exception:  # pragma: no cover - optional dependency
     _MCP_TYPES_AVAILABLE = False
     _PROTOCOL_VERSION = "2025-06-18"
@@ -84,7 +83,59 @@ class MCPServer:
         self,
         name: str = "RMCP MCP Server",
         version: str = None,
-        description: str = "R-based statistical analysis MCP server",
+        description: str = """RMCP provides 44 comprehensive statistical analysis tools through R:
+
+**Regression & Econometrics (8 tools):**
+- Linear/logistic regression with diagnostics and residual analysis
+- Panel data regression (fixed/random effects) with robust standard errors
+- Instrumental variables (2SLS) regression for causal inference
+- Vector autoregression (VAR) models for multivariate time series
+- Correlation analysis with significance testing and confidence intervals
+
+**Time Series Analysis (6 tools):**
+- ARIMA modeling with automatic order selection and forecasting
+- Time series decomposition (trend, seasonal, remainder components)
+- Stationarity testing (ADF, KPSS, Phillips-Perron tests)
+- Lag/lead variable creation and differencing transformations
+
+**Statistical Testing (5 tools):**
+- T-tests (one-sample, two-sample, paired) with effect sizes
+- ANOVA (one-way, two-way) with post-hoc comparisons
+- Chi-square tests for independence and goodness-of-fit
+- Normality tests (Shapiro-Wilk, Kolmogorov-Smirnov, Anderson-Darling)
+
+**Data Analysis & Transformation (9 tools):**
+- Comprehensive descriptive statistics with distribution analysis
+- Outlier detection using multiple methods (IQR, Z-score, Mahalanobis)
+- Data standardization (z-score, min-max, robust scaling)
+- Winsorization for outlier treatment and data cleaning
+- Professional frequency tables with percentages and cumulative statistics
+
+**Machine Learning (4 tools):**
+- K-means clustering with optimal cluster selection and visualization
+- Decision trees for classification and regression with pruning
+- Random forest models with variable importance and out-of-bag error
+
+**Professional Visualizations (6 tools):**
+- Scatter plots with trend lines, confidence bands, and grouping
+- Time series plots for single/multiple variables with forecasting
+- Histograms with density overlays and distribution fitting
+- Correlation heatmaps with hierarchical clustering
+- Box plots for distribution comparison and outlier identification
+- Comprehensive residual diagnostic plots (4-panel analysis)
+
+**File Operations (3 tools):**
+- CSV/Excel/JSON import with automatic type detection
+- Data filtering, export, and comprehensive dataset information
+- Missing value analysis and data quality reporting
+
+**Advanced Features:**
+- Formula builder: Convert natural language to R statistical formulas
+- Error recovery: Intelligent error diagnosis with suggested fixes
+- Flexible R execution: Custom R code with 80+ whitelisted packages
+- Example datasets: Built-in datasets for testing and learning
+
+All tools provide professionally formatted output with markdown tables, statistical interpretations, and inline visualizations (base64 images). Results include both raw data and formatted summaries using broom/knitr for publication-ready output.""",
     ):
         """
         Initialize the MCP server instance.
@@ -106,6 +157,7 @@ class MCPServer:
         # Transport + notification state
         self._transports: set[Transport] = set()
         self._pending_notifications: list[dict[str, Any]] = []
+        self._resource_subscribers: set[Transport] = set()
         # Registries
         self.tools = ToolsRegistry(
             on_list_changed=self._make_list_changed_callback("tools")
@@ -240,10 +292,48 @@ class MCPServer:
         Raises:
             Exception: If any startup callback fails, the exception propagates.
         """
-        logger.info(f"Starting {self.name} v{self.version}")
+        import platform
+        import sys
+
+        # Emit prominent version and system information
+        logger.info("=" * 60)
+        logger.info(f"🚀 {self.name} v{self.version}")
+        logger.info("=" * 60)
+        logger.info(
+            f"Python {sys.version.split()[0]} on {platform.system()} {platform.release()}"
+        )
+
+        # Log configuration summary
+        if self.lifespan_state.allowed_paths:
+            paths_str = ", ".join(str(p) for p in self.lifespan_state.allowed_paths[:3])
+            if len(self.lifespan_state.allowed_paths) > 3:
+                paths_str += f" (and {len(self.lifespan_state.allowed_paths) - 3} more)"
+            logger.info(f"Allowed paths: {paths_str}")
+
+        logger.info(
+            f"Access mode: {'read-only' if self.lifespan_state.read_only else 'read-write'}"
+        )
+
+        if self.lifespan_state.cache_root:
+            logger.info(f"Cache root: {self.lifespan_state.cache_root}")
+
+        # Execute startup callbacks
+        logger.info("Executing startup callbacks...")
         for callback in self._startup_callbacks:
             await callback()
-        logger.info("Server startup complete")
+
+        # Log registry summary
+        tools_count = len(getattr(self.tools, "_tools", {}))
+        resources_count = len(getattr(self.resources, "_static_resources", {})) + len(
+            getattr(self.resources, "_templates", {})
+        )
+        prompts_count = len(getattr(self.prompts, "_prompts", {}))
+
+        logger.info(
+            f"Registered: {tools_count} tools, {resources_count} resources, {prompts_count} prompts"
+        )
+        logger.info("✅ Server startup complete - ready to handle requests")
+        logger.info("=" * 60)
 
     async def shutdown(self) -> None:
         """
@@ -375,15 +465,16 @@ class MCPServer:
         """
         client_info = params.get("clientInfo", {})
         logger.info(
-            f"Initializing MCP connection with client: {client_info.get('name', 'unknown')}"
+            f"Initializing MCP connection with client: "
+            f"{client_info.get('name', 'unknown')}"
         )
         if _MCP_TYPES_AVAILABLE:
+            # Build capabilities - completion capability is not available in current MCP version
             capabilities = ServerCapabilities(
                 tools=ToolsCapability(listChanged=False),
-                resources=ResourcesCapability(subscribe=False, listChanged=False),
+                resources=ResourcesCapability(subscribe=True, listChanged=True),
                 prompts=PromptsCapability(listChanged=False),
-                logging=LoggingCapability(levels=_SUPPORTED_LOG_LEVELS),
-                completion=CompletionCapability(),
+                logging=LoggingCapability(),
             )
             initialize_result = InitializeResult(
                 protocolVersion=_PROTOCOL_VERSION,
@@ -392,16 +483,17 @@ class MCPServer:
                 instructions=self.description or None,
             )
             return initialize_result.model_dump(mode="json", exclude_none=True)
-        capabilities = {
+        # Fallback for when MCP types not available
+        capabilities_dict = {
             "tools": {"listChanged": False},
-            "resources": {"subscribe": False, "listChanged": False},
+            "resources": {"subscribe": True, "listChanged": True},
             "prompts": {"listChanged": False},
-            "logging": {"levels": _SUPPORTED_LOG_LEVELS},
+            "logging": {},
             "completion": {},
         }
         result = {
             "protocolVersion": _PROTOCOL_VERSION,
-            "capabilities": capabilities,
+            "capabilities": capabilities_dict,
             "serverInfo": {"name": self.name, "version": self.version},
         }
         if self.description:
@@ -419,13 +511,14 @@ class MCPServer:
         # Validate log level
         if level not in _SUPPORTED_LOG_LEVELS:
             raise ValueError(
-                f"Unsupported log level: {level}. Supported levels: {_SUPPORTED_LOG_LEVELS}"
+                f"Unsupported log level: {level}. "
+                f"Supported levels: {_SUPPORTED_LOG_LEVELS}"
             )
         # Map MCP levels to Python logging levels
         level_mapping = {
             "debug": logging.DEBUG,
             "info": logging.INFO,
-            "notice": logging.INFO,  # Python doesn't have NOTICE, use INFO
+            "notice": logging.INFO,  # Python doesn't have NOTICE
             "warning": logging.WARNING,
             "error": logging.ERROR,
             "critical": logging.CRITICAL,
@@ -440,6 +533,30 @@ class MCPServer:
         logger.info(f"Log level set to: {level}")
         return {}
 
+    async def _handle_resources_subscribe(self, context: Context) -> dict[str, Any]:
+        """Subscribe the current transport to resource change notifications."""
+        transport = self._current_transport()
+        if not transport:
+            raise ValueError("resources/subscribe requires an active transport")
+        self._resource_subscribers.add(transport)
+        await context.info(
+            "Transport subscribed to resource updates",
+            transport=getattr(transport, "name", "transport"),
+        )
+        return {"subscription": "resources"}
+
+    async def _handle_resources_unsubscribe(self, context: Context) -> dict[str, Any]:
+        """Unsubscribe the current transport from resource notifications."""
+        transport = self._current_transport()
+        if not transport:
+            return {}
+        self._resource_subscribers.discard(transport)
+        await context.info(
+            "Transport unsubscribed from resource updates",
+            transport=getattr(transport, "name", "transport"),
+        )
+        return {}
+
     async def _handle_completion(self, params: dict[str, Any]) -> dict[str, Any]:
         """
         Handle completion/complete request for auto-completion suggestions.
@@ -474,33 +591,32 @@ class MCPServer:
                     "type": "text",
                     "value": tool_name,
                     "label": tool_name,
-                    "detail": f"Statistical analysis tool",
+                    "detail": "Statistical analysis tool",
                 }
                 for tool_name in matching_tools[:10]  # Limit to 10 suggestions
             ]
         elif completion_type == "tool_parameters":
             # Complete parameter names for specific tools
             tool_name = ref.get("toolName")
-            if tool_name and tool_name in self.tools._registry:
-                tool_func = self.tools._registry[tool_name]
-                if hasattr(tool_func, "_rmcp_input_schema"):
-                    schema = tool_func._rmcp_input_schema
-                    properties = schema.get("properties", {})
-                    if name:
-                        matching_params = [
-                            p for p in properties.keys() if p.startswith(name)
-                        ]
-                    else:
-                        matching_params = list(properties.keys())
-                    completions = [
-                        {
-                            "type": "parameter",
-                            "value": param,
-                            "label": param,
-                            "detail": properties[param].get("description", "Parameter"),
-                        }
-                        for param in matching_params[:10]
+            if tool_name and tool_name in self.tools._tools:
+                tool_def = self.tools._tools[tool_name]
+                schema = tool_def.input_schema
+                properties = schema.get("properties", {})
+                if name:
+                    matching_params = [
+                        p for p in properties.keys() if p.startswith(name)
                     ]
+                else:
+                    matching_params = list(properties.keys())
+                completions = [
+                    {
+                        "type": "parameter",
+                        "value": param,
+                        "label": param,
+                        "detail": properties[param].get("description", "Parameter"),
+                    }
+                    for param in matching_params[:10]
+                ]
         elif completion_type == "formula":
             # Complete R formula syntax
             formula_suggestions = [
@@ -661,13 +777,20 @@ class MCPServer:
         params = request.get("params", {})
         if not isinstance(params, dict):
             params = {}
-        if method is None:
-            raise ValueError("Invalid JSON-RPC request: missing method")
+
         # Handle notifications (no response expected)
         if request_id is None:
+            if method is None:
+                # For notifications, we can't return an error, so just log and return
+                logger.error("Invalid JSON-RPC notification: missing method")
+                return None
             await self._handle_notification(method, params)
             return None
+
         try:
+            # Validate method for requests (can return error response)
+            if method is None:
+                raise ValueError("Invalid JSON-RPC request: missing method")
             progress_token = params.get("progressToken")
             tool_invocation_id = params.get("toolInvocationId")
             metadata = {}
@@ -691,6 +814,8 @@ class MCPServer:
                 )
             elif method == "tools/call":
                 tool_name = params.get("name")
+                if not isinstance(tool_name, str):
+                    raise ValueError("tools/call requires 'name' parameter")
                 arguments = params.get("arguments", {})
                 result = await self.tools.call_tool(context, tool_name, arguments)
             elif method == "resources/list":
@@ -701,7 +826,13 @@ class MCPServer:
                 )
             elif method == "resources/read":
                 uri = params.get("uri")
+                if not isinstance(uri, str):
+                    raise ValueError("resources/read requires 'uri' parameter")
                 result = await self.resources.read_resource(context, uri)
+            elif method == "resources/subscribe":
+                result = await self._handle_resources_subscribe(context)
+            elif method == "resources/unsubscribe":
+                result = await self._handle_resources_unsubscribe(context)
             elif method == "prompts/list":
                 result = await self.prompts.list_prompts(
                     context,
@@ -710,10 +841,14 @@ class MCPServer:
                 )
             elif method == "prompts/get":
                 name = params.get("name")
+                if not isinstance(name, str):
+                    raise ValueError("prompts/get requires 'name' parameter")
                 arguments = params.get("arguments", {})
                 result = await self.prompts.get_prompt(context, name, arguments)
             elif method == "logging/setLevel":
                 level = params.get("level")
+                if not isinstance(level, str):
+                    raise ValueError("logging/setLevel requires 'level' parameter")
                 result = await self._handle_set_log_level(level)
             elif method == "completion/complete":
                 result = await self._handle_completion(params)
@@ -722,10 +857,25 @@ class MCPServer:
             return {"jsonrpc": "2.0", "id": request_id, "result": result}
         except Exception as e:
             logger.error(f"Error handling request {request_id}: {e}")
+
+            # Map specific errors to appropriate JSON-RPC error codes
+            error_message = str(e)
+            if "missing method" in error_message.lower():
+                error_code = -32600  # Invalid Request
+            elif "unknown method" in error_message.lower():
+                error_code = -32601  # Method not found
+            elif (
+                "requires" in error_message.lower()
+                and "parameter" in error_message.lower()
+            ):
+                error_code = -32602  # Invalid params
+            else:
+                error_code = -32603  # Internal error
+
             return {
                 "jsonrpc": "2.0",
                 "id": request_id,
-                "error": {"code": -32603, "message": str(e)},  # Internal error
+                "error": {"code": error_code, "message": error_message},
             }
         finally:
             if request_id:
@@ -785,7 +935,18 @@ class MCPServer:
 
     async def _broadcast_notification(self, notification: dict[str, Any]) -> None:
         """Broadcast a notification to all registered transports."""
-        for transport in list(self._transports):
+        transports = list(self._transports)
+        kind = notification.get("params", {}).get("kind")
+        if kind == "resources":
+            if self._resource_subscribers:
+                transports = [
+                    transport
+                    for transport in transports
+                    if transport in self._resource_subscribers
+                ]
+            else:
+                return
+        for transport in transports:
             try:
                 await transport.send_message(dict(notification))
             except Exception as exc:  # pragma: no cover - defensive logging
@@ -831,6 +992,33 @@ class MCPServer:
                         lambda path=example_path: path.read_text(encoding="utf-8")
                     ),
                 )
+        # Discovery resources
+        self.resources.register_static_resource(
+            uri="rmcp://catalog",
+            name="Tool Catalog",
+            description="Index of tools with usage examples",
+            mime_type="text/markdown",
+        )
+        self.resources.register_static_resource(
+            uri="rmcp://env",
+            name="Environment Report",
+            description="R version, packages, and platform details",
+            mime_type="application/json",
+        )
+        self.resources.register_resource_template(
+            uri_template="rmcp://dataset/{name}",
+            name="Example Dataset",
+            description="Expose built-in sample datasets via rmcp://dataset/<name>",
+        )
+
+    def _current_transport(self) -> Transport | None:
+        """Return the transport currently handling a request, if any."""
+        info = _transport_context.get()
+        if info and isinstance(info, dict):
+            transport = info.get("transport")
+            if isinstance(transport, Transport):
+                return transport
+        return None
 
     async def _handle_notification(self, method: str, params: dict[str, Any]) -> None:
         """
@@ -858,7 +1046,59 @@ class MCPServer:
 def create_server(
     name: str = "RMCP MCP Server",
     version: str = None,
-    description: str = "R-based statistical analysis MCP server",
+    description: str = """RMCP provides 44 comprehensive statistical analysis tools through R:
+
+**Regression & Econometrics (8 tools):**
+- Linear/logistic regression with diagnostics and residual analysis
+- Panel data regression (fixed/random effects) with robust standard errors
+- Instrumental variables (2SLS) regression for causal inference
+- Vector autoregression (VAR) models for multivariate time series
+- Correlation analysis with significance testing and confidence intervals
+
+**Time Series Analysis (6 tools):**
+- ARIMA modeling with automatic order selection and forecasting
+- Time series decomposition (trend, seasonal, remainder components)
+- Stationarity testing (ADF, KPSS, Phillips-Perron tests)
+- Lag/lead variable creation and differencing transformations
+
+**Statistical Testing (5 tools):**
+- T-tests (one-sample, two-sample, paired) with effect sizes
+- ANOVA (one-way, two-way) with post-hoc comparisons
+- Chi-square tests for independence and goodness-of-fit
+- Normality tests (Shapiro-Wilk, Kolmogorov-Smirnov, Anderson-Darling)
+
+**Data Analysis & Transformation (9 tools):**
+- Comprehensive descriptive statistics with distribution analysis
+- Outlier detection using multiple methods (IQR, Z-score, Mahalanobis)
+- Data standardization (z-score, min-max, robust scaling)
+- Winsorization for outlier treatment and data cleaning
+- Professional frequency tables with percentages and cumulative statistics
+
+**Machine Learning (4 tools):**
+- K-means clustering with optimal cluster selection and visualization
+- Decision trees for classification and regression with pruning
+- Random forest models with variable importance and out-of-bag error
+
+**Professional Visualizations (6 tools):**
+- Scatter plots with trend lines, confidence bands, and grouping
+- Time series plots for single/multiple variables with forecasting
+- Histograms with density overlays and distribution fitting
+- Correlation heatmaps with hierarchical clustering
+- Box plots for distribution comparison and outlier identification
+- Comprehensive residual diagnostic plots (4-panel analysis)
+
+**File Operations (3 tools):**
+- CSV/Excel/JSON import with automatic type detection
+- Data filtering, export, and comprehensive dataset information
+- Missing value analysis and data quality reporting
+
+**Advanced Features:**
+- Formula builder: Convert natural language to R statistical formulas
+- Error recovery: Intelligent error diagnosis with suggested fixes
+- Flexible R execution: Custom R code with 80+ whitelisted packages
+- Example datasets: Built-in datasets for testing and learning
+
+All tools provide professionally formatted output with markdown tables, statistical interpretations, and inline visualizations (base64 images). Results include both raw data and formatted summaries using broom/knitr for publication-ready output.""",
 ) -> MCPServer:
     """
     Factory function to create a new MCP server instance.
