@@ -20,6 +20,7 @@ Example Usage:
 from typing import Any
 
 from ..core.schemas import formula_schema, table_schema
+from ..r_assets.loader import get_r_script
 from ..r_integration import execute_r_script_async
 from ..registries.tools import tool
 
@@ -116,7 +117,7 @@ from ..registries.tools import tool
         "required": ["coefficients", "r_squared", "n_obs", "method"],
         "additionalProperties": False,
     },
-    description="Fit linear regression model with comprehensive diagnostics",
+    description="Performs ordinary least squares (OLS) linear regression to model relationships between a dependent variable and one or more predictors. Returns coefficients with standard errors, confidence intervals, R-squared, F-statistic, and comprehensive diagnostic statistics including residuals and fitted values. Use for prediction, inference, understanding linear relationships, or testing hypotheses about continuous outcomes. Handles missing values and supports weighted observations.",
 )
 async def linear_model(context, params) -> dict[str, Any]:
     """
@@ -170,39 +171,9 @@ async def linear_model(context, params) -> dict[str, Any]:
         ... })
     """
     await context.info("Fitting linear regression model")
-    r_script = """
-    data <- as.data.frame(args$data)
-    formula <- as.formula(args$formula)
-    # Handle optional parameters
-    weights <- args$weights
-    na_action <- args$na_action %||% "na.omit"
-    # Fit model
-    if (!is.null(weights)) {
-        model <- lm(formula, data = data, weights = weights, na.action = get(na_action))
-    } else {
-        model <- lm(formula, data = data, na.action = get(na_action))
-    }
-    # Get comprehensive results
-    summary_model <- summary(model)
-    result <- list(
-        coefficients = as.list(coef(model)),
-        std_errors = as.list(summary_model$coefficients[, "Std. Error"]),
-        t_values = as.list(summary_model$coefficients[, "t value"]),
-        p_values = as.list(summary_model$coefficients[, "Pr(>|t|)"]),
-        r_squared = summary_model$r.squared,
-        adj_r_squared = summary_model$adj.r.squared,
-        f_statistic = summary_model$fstatistic[1],
-        f_p_value = pf(summary_model$fstatistic[1],
-                      summary_model$fstatistic[2],
-                      summary_model$fstatistic[3], lower.tail = FALSE),
-        residual_se = summary_model$sigma,
-        df_residual = summary_model$df[2],
-        fitted_values = as.numeric(fitted(model)),
-        residuals = as.numeric(residuals(model)),
-        n_obs = nrow(model$model),
-        method = "lm"
-    )
-    """
+
+    # Load R script from separated file
+    r_script = get_r_script("regression", "linear_model")
     try:
         result = await execute_r_script_async(r_script, params)
         await context.info(
@@ -299,7 +270,7 @@ async def linear_model(context, params) -> dict[str, Any]:
         "required": ["correlation_matrix", "method", "variables"],
         "additionalProperties": False,
     },
-    description="Comprehensive correlation analysis with significance tests",
+    description="Computes pairwise correlation matrix between numeric variables using Pearson, Spearman, or Kendall methods. Provides correlation coefficients, significance tests (p-values), and handles missing data appropriately. Use to explore relationships between variables, identify multicollinearity, or understand data structure before modeling. Pearson for linear relationships, Spearman for monotonic non-linear relationships, Kendall for small samples or ties.",
 )
 async def correlation_analysis(context, params) -> dict[str, Any]:
     """
@@ -342,76 +313,9 @@ async def correlation_analysis(context, params) -> dict[str, Any]:
         ... })
     """
     await context.info("Computing correlation matrix")
-    r_script = """
-    data <- as.data.frame(args$data)
-    variables <- args$variables
-    method <- args$method %||% "pearson"
-    use <- args$use %||% "complete.obs"
-    # Select variables if specified
-    if (!is.null(variables)) {
-        # Validate variables exist
-        missing_vars <- setdiff(variables, names(data))
-        if (length(missing_vars) > 0) {
-            stop(paste("Variables not found:", paste(missing_vars, collapse = ", ")))
-        }
-        data <- data[, variables, drop = FALSE]
-    }
-    # Select only numeric variables
-    numeric_vars <- sapply(data, is.numeric)
-    if (sum(numeric_vars) < 2) {
-        stop("Need at least 2 numeric variables for correlation analysis")
-    }
-    numeric_data <- data[, numeric_vars, drop = FALSE]
-    # Compute correlation matrix
-    cor_matrix <- cor(numeric_data, method = method, use = use)
-    # Compute significance tests and pairwise n_obs
-    n <- nrow(numeric_data)
-    cor_test_results <- list()
-    # Initialize n_obs matrix structure
-    n_obs_matrix <- matrix(0, nrow = ncol(numeric_data), ncol = ncol(numeric_data))
-    rownames(n_obs_matrix) <- names(numeric_data)
-    colnames(n_obs_matrix) <- names(numeric_data)
-    # Fill diagonal with total observations
-    diag(n_obs_matrix) <- n
-    for (i in 1:(ncol(numeric_data)-1)) {
-        for (j in (i+1):ncol(numeric_data)) {
-            var1 <- names(numeric_data)[i]
-            var2 <- names(numeric_data)[j]
-            # Filter complete cases for cor.test (cor.test doesn't accept 'use' parameter)
-            x <- numeric_data[,i]
-            y <- numeric_data[,j]
-            complete_cases <- !is.na(x) & !is.na(y)
-            n_pairwise <- sum(complete_cases)
-            # Store pairwise n_obs in matrix
-            n_obs_matrix[i, j] <- n_pairwise
-            n_obs_matrix[j, i] <- n_pairwise
-            test_result <- cor.test(x[complete_cases], y[complete_cases], method = method)
-            cor_test_results[[paste(var1, var2, sep = "_")]] <- list(
-                correlation = test_result$estimate,
-                p_value = test_result$p.value,
-                conf_int_lower = if (!is.null(test_result$conf.int)) test_result$conf.int[1] else NA,
-                conf_int_upper = if (!is.null(test_result$conf.int)) test_result$conf.int[2] else NA
-            )
-        }
-    }
-    # Convert correlation matrix to nested list structure
-    cor_list <- list()
-    for (var1 in names(numeric_data)) {
-        cor_list[[var1]] <- as.list(setNames(cor_matrix[var1, ], names(numeric_data)))
-    }
-    # Convert n_obs matrix to nested list structure
-    n_obs_list <- list()
-    for (var1 in names(numeric_data)) {
-        n_obs_list[[var1]] <- as.list(setNames(n_obs_matrix[var1, ], names(numeric_data)))
-    }
-    result <- list(
-        correlation_matrix = cor_list,
-        significance_tests = cor_test_results,
-        method = method,
-        n_obs = n_obs_list,
-        variables = names(numeric_data)
-    )
-    """
+
+    # Load R script from separated file
+    r_script = get_r_script("regression", "correlation_analysis")
     try:
         result = await execute_r_script_async(r_script, params)
         await context.info(
@@ -539,72 +443,14 @@ async def correlation_analysis(context, params) -> dict[str, Any]:
         ],
         "additionalProperties": False,
     },
-    description="Fit generalized linear model (logistic regression)",
+    description="Fits generalized linear models (GLM) including logistic regression for binary outcomes, Poisson regression for count data, and other exponential family distributions. Returns coefficients, odds ratios (for logistic), AIC/BIC for model comparison, and diagnostic statistics. Use for binary classification, count modeling, or when dependent variable doesn't follow normal distribution. Includes prediction probabilities and confusion matrix for classification tasks.",
 )
 async def logistic_regression(context, params) -> dict[str, Any]:
     """Fit logistic regression model."""
     await context.info("Fitting logistic regression model")
-    r_script = """
-    data <- as.data.frame(args$data)
-    formula <- as.formula(args$formula)
-    family <- args$family %||% "binomial"
-    link <- args$link %||% "logit"
-    # Prepare family specification
-    if (family == "binomial") {
-        family_spec <- binomial(link = link)
-    } else {
-        family_spec <- get(family)()
-    }
-    # Fit GLM
-    model <- glm(formula, data = data, family = family_spec)
-    summary_model <- summary(model)
-    # Additional diagnostics for logistic regression
-    if (family == "binomial") {
-        # Odds ratios
-        odds_ratios <- exp(coef(model))
-        # McFadden's R-squared
-        ll_null <- logLik(glm(update(formula, . ~ 1), data = data, family = family_spec))
-        ll_model <- logLik(model)
-        mcfadden_r2 <- 1 - (ll_model / ll_null)
-        # Predicted probabilities
-        predicted_probs <- fitted(model)
-        predicted_classes <- ifelse(predicted_probs > 0.5, 1, 0)
-        # Confusion matrix (if binary outcome)
-        actual <- model.response(model.frame(model))
-        if (all(actual %in% c(0, 1))) {
-            confusion <- table(actual, predicted_classes)
-            accuracy <- sum(diag(confusion)) / sum(confusion)
-        } else {
-            confusion <- NULL
-            accuracy <- NULL
-        }
-    }
-    result <- list(
-        coefficients = as.list(coef(model)),
-        std_errors = as.list(summary_model$coefficients[, "Std. Error"]),
-        z_values = as.list(summary_model$coefficients[, "z value"]),
-        p_values = as.list(summary_model$coefficients[, "Pr(>|z|)"]),
-        deviance = model$deviance,
-        null_deviance = model$null.deviance,
-        aic = AIC(model),
-        bic = BIC(model),
-        fitted_values = as.numeric(fitted(model)),
-        residuals = as.numeric(residuals(model, type = "deviance")),
-        n_obs = nobs(model),
-        family = family,
-        link = link
-    )
-    # Add logistic-specific results
-    if (family == "binomial") {
-        result$odds_ratios <- as.list(odds_ratios)
-        result$mcfadden_r_squared <- as.numeric(mcfadden_r2)
-        result$predicted_probabilities <- predicted_probs
-        if (!is.null(accuracy)) {
-            result$accuracy <- accuracy
-            result$confusion_matrix <- as.list(as.data.frame.matrix(confusion))
-        }
-    }
-    """
+
+    # Load R script from separated file
+    r_script = get_r_script("regression", "logistic_regression")
     try:
         result = await execute_r_script_async(r_script, params)
         await context.info(
