@@ -5,7 +5,7 @@ Comprehensive time series modeling and forecasting capabilities.
 
 from typing import Any
 
-from ..core.schemas import table_schema
+from ..r_assets.loader import get_r_script
 from ..r_integration import execute_r_script_async
 from ..registries.tools import tool
 
@@ -125,61 +125,13 @@ from ..registries.tools import tool
         ],
         "additionalProperties": False,
     },
-    description="Fit ARIMA time series model with forecasting",
+    description="Fits ARIMA (AutoRegressive Integrated Moving Average) models for time series forecasting and analysis. Automatically selects optimal model parameters or uses specified orders, generates point forecasts with confidence intervals, and provides model diagnostics including AIC/BIC. Use for forecasting future values, understanding temporal patterns, or modeling autocorrelated data. Handles both non-seasonal and seasonal ARIMA models with integrated differencing for non-stationary series.",
 )
 async def arima_model(context, params) -> dict[str, Any]:
     """Fit ARIMA model and generate forecasts."""
     await context.info("Fitting ARIMA time series model")
-    r_script = """
-    # Install required packages
-    library(forecast)
-    # Prepare data
-    rmcp_progress("Preparing time series data")
-    values <- args$data$values
-    # Convert to time series
-    if (!is.null(args$data$dates)) {
-        dates <- as.Date(args$data$dates)
-        ts_data <- ts(values, frequency = 12)  # Assume monthly by default
-    } else {
-        ts_data <- ts(values, frequency = 12)
-    }
-    # Fit ARIMA model with progress reporting
-    rmcp_progress("Fitting ARIMA model", 20, 100)
-    if (!is.null(args$order)) {
-        if (!is.null(args$seasonal)) {
-            model <- Arima(ts_data, order = args$order, seasonal = args$seasonal)
-        } else {
-            model <- Arima(ts_data, order = args$order)
-        }
-    } else {
-        # Auto ARIMA (can be slow for large datasets)
-        rmcp_progress("Running automatic ARIMA model selection", 30, 100)
-        model <- auto.arima(ts_data)
-    }
-    rmcp_progress("ARIMA model fitted successfully", 70, 100)
-    # Generate forecasts
-    rmcp_progress("Generating forecasts", 80, 100)
-    forecast_periods <- args$forecast_periods %||% 12
-    forecasts <- forecast(model, h = forecast_periods)
-    rmcp_progress("Extracting model results", 95, 100)
-    # Extract results
-    result <- list(
-        model_type = "ARIMA",
-        order = arimaorder(model),
-        coefficients = as.list(coef(model)),
-        aic = AIC(model),
-        bic = BIC(model),
-        loglik = logLik(model)[1],
-        sigma2 = model$sigma2,
-        fitted_values = as.numeric(fitted(model)),
-        residuals = as.numeric(residuals(model)),
-        forecasts = as.numeric(forecasts$mean),
-        forecast_lower = as.numeric(forecasts$lower[,2]),  # 95% CI
-        forecast_upper = as.numeric(forecasts$upper[,2]),
-        accuracy = accuracy(model),
-        n_obs = length(values)
-    )
-    """
+
+    r_script = get_r_script("timeseries", "arima_model")
     try:
         result = await execute_r_script_async(r_script, params, context)
         await context.info(
@@ -265,34 +217,13 @@ async def arima_model(context, params) -> dict[str, Any]:
         ],
         "additionalProperties": False,
     },
-    description="Decompose time series into trend, seasonal, and remainder components",
+    description="Decomposes time series into constituent components: trend (long-term movement), seasonal (repeating patterns), and remainder (irregular fluctuations). Supports both additive and multiplicative decomposition methods. Use to understand underlying patterns, identify seasonal effects, detect structural changes, or prepare data for forecasting. Essential for exploratory time series analysis and identifying appropriate modeling approaches.",
 )
 async def decompose_timeseries(context, params) -> dict[str, Any]:
     """Decompose time series into components."""
     await context.info("Decomposing time series")
-    r_script = """
-    values <- args$data$values
-    frequency <- args$frequency %||% 12
-    decomp_type <- args$type %||% "additive"
-    # Create time series
-    ts_data <- ts(values, frequency = frequency)
-    # Decompose
-    if (decomp_type == "multiplicative") {
-        decomp <- decompose(ts_data, type = "multiplicative")
-    } else {
-        decomp <- decompose(ts_data, type = "additive")
-    }
-    # Handle NA values properly for JSON - use I() to preserve arrays
-    result <- list(
-        original = I(as.numeric(decomp$x)),
-        trend = I(as.numeric(decomp$trend)),
-        seasonal = I(as.numeric(decomp$seasonal)),
-        remainder = I(as.numeric(decomp$random)),
-        type = decomp_type,
-        frequency = frequency,
-        n_obs = length(values)
-    )
-    """
+
+    r_script = get_r_script("timeseries", "decompose_timeseries")
     try:
         result = await execute_r_script_async(r_script, params, context)
         await context.info("Time series decomposed successfully")
@@ -366,44 +297,13 @@ async def decompose_timeseries(context, params) -> dict[str, Any]:
         ],
         "additionalProperties": False,
     },
-    description="Test time series for stationarity (ADF, KPSS, Phillips-Perron)",
+    description="Tests time series for stationarity using Augmented Dickey-Fuller (ADF), Kwiatkowski-Phillips-Schmidt-Shin (KPSS), or Phillips-Perron tests. Stationarity is required for many time series models like ARIMA. Returns test statistics, p-values, critical values, and clear interpretation. Use before time series modeling to determine if differencing is needed, or to verify model assumptions. ADF tests for unit roots, KPSS tests trend stationarity.",
 )
 async def stationarity_test(context, params) -> dict[str, Any]:
     """Test time series stationarity."""
     await context.info("Testing time series stationarity")
-    r_script = """
-    library(tseries)
-    values <- args$data$values
-    test_type <- args$test %||% "adf"
-    ts_data <- ts(values)
-    if (test_type == "adf") {
-        test_result <- adf.test(ts_data)
-        test_name <- "Augmented Dickey-Fuller"
-    } else if (test_type == "kpss") {
-        test_result <- kpss.test(ts_data)
-        test_name <- "KPSS"
-    } else if (test_type == "pp") {
-        test_result <- pp.test(ts_data)
-        test_name <- "Phillips-Perron"
-    }
-    # Handle critical values properly - some tests might not have them
-    critical_vals <- if (is.null(test_result$critical) || length(test_result$critical) == 0) {
-        # Return empty named list to ensure it's treated as object, not array
-        structure(list(), names = character(0))
-    } else {
-        as.list(test_result$critical)
-    }
-    result <- list(
-        test_name = test_name,
-        test_type = test_type,
-        statistic = as.numeric(test_result$statistic),
-        p_value = test_result$p.value,
-        critical_values = critical_vals,
-        alternative = test_result$alternative,
-        is_stationary = if (test_type == "kpss") test_result$p.value > 0.05 else test_result$p.value < 0.05,
-        n_obs = length(values)
-    )
-    """
+
+    r_script = get_r_script("timeseries", "stationarity_test")
     try:
         result = await execute_r_script_async(r_script, params, context)
         await context.info(
